------------------------------------------------------------------------------
-- MakeMeGetMyMain - Alarms you when something happens to your character
------------------------------------------------------------------------------
-- GUI.lua - Interface
--
-- Author: Expelliarm5s / April 2021 / All Rights Reserved
--
-- Version 1.1.10
------------------------------------------------------------------------------
-- luacheck: ignore 212 globals DLAPI UISpecialFrames StaticPopupDialogs StaticPopup_Show StaticPopup_Hide


local addonName, addon = ...
local GUI = addon:NewModule("GUI", "AceEvent-3.0", "AceConsole-3.0")
-- local L = LibStub("AceLocale-3.0"):GetLocale(addonName)
local AceGUI = LibStub("AceGUI-3.0")
local private = {}
------------------------------------------------------------------------------

private.display = nil

function GUI:DebugPrintf(...)
	if addon.isDebug then
		local status, res = pcall(format, ...)
		if status then
			addon:DebugLog("GUI~" .. res)
		end
	end
end

function GUI:Close()
	if private.display and private.display.frame and private.display.frame.obj and private.display.frame.obj.Fire then
		private.display.frame.obj:Fire("OnClose")
	end
end

function GUI:Fire(m,i)
	GUI:DebugPrintf("GUI:Fire(%s, %s)", tostring(m), tostring(i))

	if not private.display then
		local display = AceGUI:Create("Frame")
		display.frame:SetFrameStrata("MEDIUM")
		display.frame:SetWidth(900)
		display.frame:SetHeight(800)

		GUI:DebugPrintf("  created the new Frame widget %s", tostring(display))

		local title = "|cFF33FF99" .. addon.METADATA.NAME .. " (" .. addon.METADATA.VERSION .. ")|r"
		display:SetTitle(title)
		display:SetLayout("Fill")
		display:SetStatusText("--")
		display:EnableResize(true)

		display:SetCallback("OnClose", function(_display)
			AceGUI:Release(_display)
			private.display = nil
			_G[addonName .. "Frame"] = nil
			-- collectgarbage()
		end)

		private.content = AceGUI:Create("ScrollFrame")
		private.content:SetLayout("Flow")
		display:AddChildren(private.content)

		if addon.db.global.closeOnEscape then
			_G[addonName .. "Frame"] = display.frame
			tinsert(UISpecialFrames, addonName .. "Frame")
		else
			_G[addonName .. "Frame"] = nil
		end

		private.display = display
	end

	local b = AceGUI:Create("Icon")
	b:SetLabel(m)
	local icon = "INTERFACE\\ICONS\\INV_Misc_QuestionMark"
	if addon[m].ModuleIcon then
		icon = addon[m].ModuleIcon()
	end
	if i then
		icon = i
	end
	b:SetImage(icon)
	b:SetImageSize(180,180)
	private.content:AddChildren(b)

	if addon.db.global.windowTransparent then
		private.display.frame:SetBackdropColor(0,0,0,0)
	else
		private.display.frame:SetBackdropColor(0.2,0.2,0.2,1)
	end

	-- relocate to allow looting
	if private.display.frame:GetParent() then
		if not addon.db.global.windowFullMode then
			private.display.frame:SetPoint("TOP", private.display.frame:GetParent(), "TOP", 0, -50)
			private.display.frame:SetPoint("BOTTOM", private.display.frame:GetParent(), "CENTER", 0, 120)
		else
			private.display.frame:SetPoint("TOP", private.display.frame:GetParent(), "TOP", 0, -150)
			private.display.frame:SetPoint("BOTTOM", private.display.frame:GetParent(), "BOTTOM", 0, 150)
		end
	end

	private.display:Show()

	addon.Options.PlaySound(addon.db.global["Sound" .. m])
end

------------------------------------------------------------------------------
-- Export/Import-UI

GUI.exportDisplay = nil

------------------------------------------------------------------------------
-- Create/Show Interface

StaticPopupDialogs[addonName .. "_ERROR"] = {
	text = "ERROR! Please review error message in the chat window!",
	button1 = ACCEPT,
	button2 = CANCEL,
	timeout = 0,
	whileDead = 1,
	hideOnEscape = 1,
	multiple = 0,
}

function GUI:ShowError(msg)
	GUI:DebugPrintf("ShowError()")

	if not msg then
		return
	end
	addon:Printf(msg)
	StaticPopup_Show(addonName .. "_ERROR")
end

StaticPopupDialogs[addonName .. "_IMPORT"] = {
	text = "Are you SURE to import ?",
	button1 = ACCEPT,
	button2 = CANCEL,
	timeout = 0,
	whileDead = 1,
	hideOnEscape = 1,
	multiple = 0,
}

function GUI:BtnImport()
	GUI:DebugPrintf("BtnImport()")

	local ebData
	if private.editframe then
		ebData = private.editframe:GetText()
	end

	if not ebData then
		return
	end

	if addon.db.global.promptForImport then
		StaticPopupDialogs[addonName .. "_IMPORT"].OnAccept = function()
			addon.GUI:DebugPrintf(addonName .. "_IMPORT")
			StaticPopup_Hide(addonName .. "_IMPORT")

			local result = addon.Data:Import(ebData)
			if result then
				GUI:ShowError(result)
			end
		end
		StaticPopup_Show(addonName .. "_IMPORT")
	else
		local result = addon.Data:Import(ebData)
		if result then
			GUI:ShowError(result)
		end
	end
end

function GUI:BtnExport()
	GUI:DebugPrintf("BtnExport()")

	local msg, ebData = addon.Data:Export()

	if ebData and private.editframe then
		private.editframe:SetText(ebData)
	else
		if msg then
			GUI:ShowError(msg)
		end
	end
end

-- create and return button/other widget
function GUI.AddButton(widgetType, label, status)
	GUI:DebugPrintf("AddButton %s, %s", widgetType, label)
	if not status then
		widgetType, label, status = "Button", widgetType, label
	end
	local button = assert(AceGUI:Create(widgetType))
	if button.SetText then
		button:SetText(tostring(label))
	end
	button:SetCallback("OnEnter", function() GUI:SetStatusLine(status) end)
	button:SetCallback("OnLeave", function() GUI:SetStatusLine("") end)
	return button
end

function GUI:Export()
	GUI:DebugPrintf("Load()")

	-- create main window
	if not GUI.exportDisplay then
		local display = AceGUI:Create("Frame")
		-- addon.db.global.ui = addon.db.global.ui or {}
		-- display:SetStatusTable(addon.db.global.ui)
		display.frame:SetFrameStrata("MEDIUM")

		GUI:DebugPrintf("  created the new widget %s", tostring(display))

		local title = "|cFF33FF99" .. addon.METADATA.NAME .. " (" .. addon.METADATA.VERSION .. ")|r"
		display:SetTitle(title)

		display:SetLayout("Fill")
		display:SetStatusText("--")
		display:EnableResize(true)
		display:SetCallback("OnClose", function(_display)
			GUI:DebugPrintf("OnClose for widget %s", tostring(_display))
			AceGUI:Release(_display)
			GUI.exportDisplay = nil
			-- collectgarbage()
		end)

		GUI.exportDisplay = display
	end

	if not GUI.exportDisplay then
		return
	end

	if #GUI.exportDisplay.children > 0 then
		GUI.exportDisplay:ReleaseChildren()
	end

	local grp = AceGUI:Create("SimpleGroup")
	grp:SetLayout("Flow")

	-- import button
	local btnImport = AceGUI:Create("Button")
	btnImport:SetText("IMPORT")
	btnImport:SetRelativeWidth(1 / 2)
	btnImport:SetCallback("OnEnter", function() GUI:SetStatusLine("Import Data") end)
	btnImport:SetCallback("OnLeave", function() GUI:SetStatusLine("") end)
	btnImport:SetCallback("OnClick", function()
		GUI:DebugPrintf("BtnImport_OnClick()")
		GUI:BtnImport()
	end)
	grp:AddChild(btnImport)

	-- export button
	local btnExport = AceGUI:Create("Button")
	btnExport:SetText("EXPORT")
	btnExport:SetRelativeWidth(1 / 2)
	btnExport:SetCallback("OnEnter", function() GUI:SetStatusLine("Import Data") end)
	btnExport:SetCallback("OnLeave", function() GUI:SetStatusLine("") end)
	btnExport:SetCallback("OnClick", function()
		GUI:DebugPrintf("BtnExport_OnClick()")
		GUI:BtnExport()
	end)
	grp:AddChild(btnExport)

	local editGrp = AceGUI:Create("InlineGroup")
	editGrp:SetFullWidth(true)
	editGrp:SetFullHeight(true)
	editGrp:SetLayout("Fill")

	local editFrame = AceGUI:Create("MMGMMEditScrollFrame")
	editFrame:SetLayout("Fill")
	private.editframe = editFrame.content

	editGrp:AddChild(editFrame)
	grp:AddChild(editGrp)
	GUI.exportDisplay:AddChild(grp)

	GUI.exportDisplay:Show()

	return GUI.exportDisplay
end

-----------------------------------------------------------------------------
-- Status Line

function GUI:SetStatusLine(txt)
	if GUI.exportDisplay then
		GUI.exportDisplay:SetStatusText(txt)
	end
end

-- EOF
